// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "ShaderLoader.hpp"
#include "assert/assert.hpp"
#include "asset/AssetManager.hpp"
#include "game/Game.hpp"

using namespace Dawn;

const std::string ShaderLoader::ASSET_TYPE = "shader";

ShaderLoader::ShaderLoader(
  const std::shared_ptr<AssetManager> assetManager,
  const std::string name
) :
  AssetLoader(assetManager, name),
  state(ShaderLoaderState::INITIAL),
  shader(std::make_shared<ShaderProgram2>())
{
}

void ShaderLoader::updateAsync() {
}

void ShaderLoader::updateSync() {
  if(state != ShaderLoaderState::INITIAL) return;
  this->state = ShaderLoaderState::LOADING;
  assertFalse(loaded, "ShaderLoader already loaded.");

  // Shorthand.
  auto sm = this->getAssetManager()->getGame()->shaderManager;

  // Load the shader string
  Slang::ComPtr<IBlob> diagnostics;
  module = sm->session->loadModule(
    this->name.c_str(),
    diagnostics.writeRef()
  );

  // Get list of entry points and create components
  int32_t definedEntryPointCount = module->getDefinedEntryPointCount();
  IComponentType** components = new IComponentType*[definedEntryPointCount + 1];
  int32_t componentCount = 0;
  components[componentCount++] = module;// First component is module.

  // Get the entry point info and append to components list.
  for(auto i = 0; i < definedEntryPointCount; i++) {
    Slang::ComPtr<IEntryPoint> ep;
    auto result = module->getDefinedEntryPoint(i, ep.writeRef());
    if(result != SLANG_OK) assertUnreachable("Failed to get entry point.");
    components[componentCount++] = ep;
  }

  // Create the composite component type
  sm->session->createCompositeComponentType(
    components,
    componentCount,
    program.writeRef(),
    diagnostics.writeRef()
  );
  if(diagnostics) {
    assertUnreachable("%s\n", (const char*) diagnostics->getBufferPointer());
    return;
  }

  // Link the program.
  auto result = program->link(linkedProgram.writeRef(), diagnostics.writeRef());
  if(diagnostics) {
    assertUnreachable("%s\n", (const char*) diagnostics->getBufferPointer());
    return;
  }

  // Create the shader program.
  Slang::ComPtr<IBlob> blob;
  slang::ProgramLayout* layout = program->getLayout();
  std::vector<std::shared_ptr<ShaderEntry>> shaderEntries;
  for(auto i = 0; i < definedEntryPointCount; i++) {
    // Get the code
    auto result = linkedProgram->getEntryPointCode(
      i,
      0,
      blob.writeRef(),
      diagnostics.writeRef()
    );

    if(diagnostics) {
      assertUnreachable("%s\n", (const char*) diagnostics->getBufferPointer());
    }

    // Get the stage information
    auto entryPointReflection = layout->getEntryPointByIndex(0);
    auto stage = entryPointReflection->getStage();

    // Create the shader entry
    auto shaderEntry = std::make_shared<ShaderEntry>();
    shaderEntry->init(
      stage,
      std::string((const char*)blob->getBufferPointer())
    );

    // Add to the list
    shaderEntries.push_back(shaderEntry);
  }

  // Create the shader program.
  shader->init(shaderEntries);

  // Finished loading.
  delete [] components;
  this->state = ShaderLoaderState::LOADED;
  this->loaded = true;
}

std::string ShaderLoader::getAssetType() const {
  return ShaderLoader::ASSET_TYPE;
}

std::shared_ptr<IShaderProgram2> ShaderLoader::getShader() {
  assertNotNull(shader, "ShaderLoader shader is null.");
  return shader;
}

ShaderLoader::~ShaderLoader() {
  shader = nullptr;

  if(linkedProgram) {
    linkedProgram->release();
    linkedProgram = nullptr;
  }
}