// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "SceneLoader.hpp"
#include "game/Game.hpp"
#include "assert/assert.hpp"
#include "scene/Scene.hpp"
#include "util/JSON.hpp"
#include "PrefabLoader.hpp"

using namespace Dawn;

const std::string SceneLoader::ASSET_TYPE = "scene";

SceneLoader::SceneLoader(
  const std::shared_ptr<AssetManager> assetManager,
  const std::string name
) :
  LoaderForSceneItems(assetManager, name),
  state(SceneLoaderState::INITIAL)
{
  this->typetest = this->getAssetType();
}

void SceneLoader::updateAsync() {
  switch(this->state) {
    case SceneLoaderState::INITIAL:
      this->jsonLoader = getAssetManager()->get<JSONLoader>(this->name);
      this->state = SceneLoaderState::LOADING_JSON;
      break;
    
    case SceneLoaderState::LOADING_JSON:
      assertNotNull(this->jsonLoader, "JSON Loader is NULL?");
      if(!this->jsonLoader->loaded) return;
      this->ctx->data = this->jsonLoader->data;
      this->setupDependencies();
      this->state = SceneLoaderState::LOADING_DEPENDENCIES;
      break;

    case SceneLoaderState::LOADING_DEPENDENCIES:
      assertTrue(this->jsonLoader->loaded, "JSON loader not loaded?");
      // Check if all dependencies are loaded.
      for(auto &asset : ctx->assets) {
        if(!asset.second->loaded) return;
      }
      ctx->currentScene = std::make_shared<Scene>(
        this->getAssetManager()->getGame()
      );
      this->state = SceneLoaderState::PENDING_STAGE;
      break;
    
    default:
      break;
  }
}

void SceneLoader::updateSync() {
  if(this->state != SceneLoaderState::PENDING_STAGE) return;
  assertNotNull(this->jsonLoader, "JSON Loader is NULL?");
  assertNotNull(ctx, "SceneLoadContext is NULL?");
  assertTrue(this->jsonLoader->loaded, "JSON loader not loaded?");

  auto &data = this->jsonLoader->data;
  if(data.contains("items")) {
    // Create the scene items
    for(auto &item : data["items"].items()) {
      auto &itemName = item.key();
      auto &itemData = item.value();
      auto sceneItem = ctx->currentScene->createSceneItem();
      ctx->items[itemName] = sceneItem;
    }

    // Add components to each scene item
    for(auto &item : data["items"].items()) {
      auto &itemName = item.key();
      auto &itemData = item.value();
      auto sceneItem = ctx->items[itemName];
      sceneItem->name = itemName;

      std::shared_ptr<SceneLoadContext> itemCtx;
      if(itemData.contains("prefab")) {
        auto prefabLoader = ctx->getAsset<PrefabLoader>(
          itemData["prefab"].get<std::string>()
        );
        assertNotNull(prefabLoader, "Prefab loader not found");
        assertTrue(prefabLoader->loaded, "Prefab loader not loaded");
        assertNotNull(
          prefabLoader->jsonLoader,
          "Prefab JSON loader not found"
        );
        assertTrue(
          prefabLoader->jsonLoader->loaded,
          "Prefab JSON loader not loaded"
        );
        itemCtx = prefabLoader->ctx;
        itemCtx->data = json(prefabLoader->jsonLoader->data);
        itemCtx->parent = this->ctx;
        JSON::mergeObjectsRecursive(itemCtx->data, itemData);
      } else {
        itemCtx = this->ctx;
        itemCtx->data = json(itemData);
      }

      sceneItem->load(itemCtx);
    }
  }
  
  this->jsonLoader = nullptr;
  this->state = SceneLoaderState::DONE;
  this->loaded = true;
}

std::string SceneLoader::getAssetType() const {
  return SceneLoader::ASSET_TYPE;
}

std::shared_ptr<Scene> SceneLoader::getScene() {
  assertNotNull(ctx, "Context is NULL?");
  assertNotNull(ctx->currentScene, "Scene not loaded?");
  return ctx->currentScene;
}

SceneLoader::~SceneLoader() {
}