// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "TilesetGrid.hpp"
#include "assert/assert.hpp"

using namespace Dawn;

TilesetGrid::TilesetGrid(
  const int32_t columns,
  const int32_t rows,
  const int32_t width,
  const int32_t height,
  const int32_t gapX,
  const int32_t gapY,
  const int32_t borderX,
  const int32_t borderY
) {
  assertTrue(columns >= 1, "Columns must be greater than or equal to 1.");
  assertTrue(rows >= 1, "Rows must be greater than or equal to 1.");
  // assertTrue(width >= 1, "Width must be greater than or equal to 1.");
  // assertTrue(height >= 1, "Height must be greater than or equal to 1.");
  assertTrue(gapX >= 0, "GapX must be greater than or equal to 0.");
  assertTrue(gapY >= 0, "GapY must be greater than or equal to 0.");
  assertTrue(borderX >= 0, "BorderX must be greater than or equal to 0.");
  assertTrue(borderY >= 0, "BorderY must be greater than or equal to 0.");
  assertTrue(
    width >= (columns + (gapX * columns) + borderX + borderX),
    "Width must allow for at least 1px per column."
  );
  assertTrue(
    height >= (rows + (gapY * rows) + borderY + borderY),
    "Height must allow for at least 1px per row."
  );

  this->rows = rows;
  this->columns = columns;

  // Calculate division sizes (pixels)
  float_t divX = (
    (float_t)width - ((float_t)borderX * 2.0f) -
    ((float_t)gapX * ((float_t)columns - 1))
  ) / columns;

  float_t divY = (
    (float_t)height - ((float_t)borderY * 2.0f) -
    ((float_t)gapY * ((float_t)rows - 1))
  ) / rows;

  // Calculate the division sizes (units)
  float_t tdivX = divX / (float_t)width;
  float_t tdivY = divY / (float_t)height;

  for(int32_t y = 0; y < rows; y++) {
    for(int32_t x = 0; x < columns; x++) {
      glm::vec4 tile;
      tile[0] = (borderX + (divX * x) + (gapX * x)) / width;
      tile[2] = tile[0] + tdivX;
          
      tile[1] = (borderY + (divY * y) + (gapY * y)) / height;
      tile[3] = tile[1] + tdivY;
      this->tiles.push_back(tile);
    }
  }
}

glm::vec4 TilesetGrid::getTile(
  const int32_t column,
  const int32_t row
) {
  assertTrue(row >= 0 && row < this->rows, "Row out of bounds.");
  assertTrue(column >= 0 && column < this->columns, "Column out of bounds.");
  return this->tiles[(row * this->columns) + column];
}