// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "LocaleManager.hpp"
#include "assert/assert.hpp"
#include "game/Game.hpp"
#include "util/String.hpp"

using namespace Dawn;

LocaleManager::LocaleManager() {
}

void LocaleManager::init(const std::shared_ptr<Game> &game) {
  assertNotNull(game, "Game cannot be null.");
  this->game = game;

  languageAsset = game->assetManager->get<JSONLoader>("en.json");
}

std::string LocaleManager::getString(
  const std::string &key,
  const std::unordered_map<std::string, std::string> replacements
) {
  assertNotNull(languageAsset, "Language asset cannot be null.");
  assertTrue(
    languageAsset->loaded,
    "Language asset must be loaded before attempting to get a string."
  );

  // Key comes in like "main_menu.new_game", which would be;
  // data["main_menu"]["new_game"], so we need to split by dot and iterate.
  auto keyParts = String::split(key, ".");
  int32_t currentPart = 0;
  json dataCurrent = languageAsset->data;

  while(currentPart < keyParts.size()) {
    auto &part = keyParts[currentPart];
    assertTrue(
      dataCurrent.contains(part),
      "Key '%s' does not exist in the language asset.", key.c_str()
    );
    dataCurrent = dataCurrent[part];
    currentPart++;
  }

  // For each replacement, replace "{{ key }}" with the value.
  std::string output = dataCurrent.get<std::string>();// overrides?
  for(auto &replacement : replacements) {
    std::string key = "{{ " + replacement.first + " }}";
    size_t pos = 0;
    while((pos = output.find(key, pos)) != std::string::npos) {
      output.replace(pos, key.length(), replacement.second);
      pos += replacement.second.length();
    }
  }

  return output;
}

LocaleManager::~LocaleManager() {
  languageAsset = nullptr;
  game.reset();
}
