// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "game/Game.hpp"
#include "scene/SceneItem.hpp"
#include "event/Event.hpp"
#include "time/event/IntervalEvent.hpp"
#include "time/event/TimeoutEvent.hpp"
#include "util/Flag.hpp"

namespace Dawn {
  enum class SceneState : flag_t {
    INITIAL = FLAG(0),
    INITIALIZED = FLAG(1),
    DEINITIALIZED = FLAG(2),
  };
  
  class Scene final : public std::enable_shared_from_this<Scene> {
    private:
      std::weak_ptr<Game> game;
      std::vector<std::shared_ptr<SceneItem>> sceneItems;
      std::vector<std::shared_ptr<SceneItem>> sceneItemsToRemove;
      bool_t paused = false;
      flag_t state = (flag_t)SceneState::INITIAL;

    public:
      Event<float_t> onUnpausedUpdate;
      Event<float_t> onPausedUpdate;
      Event<> onNextFrame;
      TimeoutEvent onTimeout;
      IntervalEvent onInterval;

      /**
       * Constructs a scene object.
       * 
       * @param game Game that initialized this scene.
       */
      Scene(const std::shared_ptr<Game> &game);

      /**
       * Stages all of the scene items on the scene.
       */
      void init();

      /**
       * Called when the scene is supposed to be deinitialized.
       */
      void deinit();

      /**
       * Called by the game every frame that the scene is set as the currently
       * active scene. This may not be sequential, for example a scene can be
       * active, then switched to a different scene temporarily, and then can be
       * switched back to the active one.
       */
      void update();

      /**
       * Returns a copy of the game instance that this scene belongs to.
       * 
       * @return Shared pointer to the game instance.
       */
      std::shared_ptr<Game> getGame();

      /**
       * Creates a scene item that belongs to this scene.
       * 
       * @return Pointer to the new scene item.
       */
      std::shared_ptr<SceneItem> createSceneItem();

      /**
       * Removes a scene item from the scene. This remove will happen at the 
       * start of the next scene update.
       * 
       * @param item Scene item to remove.
       */
      void removeItem(const std::shared_ptr<SceneItem> item);

      /**
       * Returns a list of scene components that match the given type.
       * 
       * @return List of scene components matching the type.
       */
      template<class T>
      std::vector<std::shared_ptr<T>> findComponents() {
        std::vector<std::shared_ptr<T>> components;
        for(auto item : sceneItems) {
          auto component = item->getComponent<T>();
          if(component) components.push_back(component);
        }
        return components;
      }

      /**
       * Destroys the scene object and cleans up all of its children.
       */
      virtual ~Scene();
  };
}