// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/SceneComponent.hpp"

namespace Dawn {
  class SceneItem;

  class SceneItemComponents {
    protected:
      std::vector<std::shared_ptr<SceneComponent>> components;

      /**
       * Shorthand way of allowing this class to retreive itself as a SceneItem.
       * 
       * @return This as a shared pointer to a SceneItem.
       */
      virtual std::shared_ptr<SceneItem> sceneItemComponentsSelf() = 0;

    public:
      SceneItemComponents();

      /**
       * Removes a component from this item.
       * 
       * @param component Component to remove.
       */
      void removeComponent(const std::shared_ptr<SceneComponent> component);

      /**
       * Adds a component to this item. You are given a shared pointer back but
       * you are not intended to take responsibility for the component.
       * 
       * @return Shared pointer to the created component.
       */
      template<class T>
      std::shared_ptr<T> addComponent() {
        //Create the component and add it.
        std::shared_ptr<T> component = std::make_shared<T>();
        this->components.push_back(
          static_pointer_cast<SceneComponent>(component)
        );
        return component;
      }

      /**
       * Returns a list of components that match the given type.
       * 
       * @return List of components matching the type.
       */
      template<class T>
      std::shared_ptr<T> getComponent() {
        for(auto component : this->components) {
          auto cast = std::dynamic_pointer_cast<T>(component);
          if(cast) return cast;
        }
        return nullptr;
      }

      virtual ~SceneItemComponents();
  };
}